@@ -1,6 +1,7 @@
 /* GObject introspection: Repository
  *
  * Copyright (C) 2005 Matthias Clasen
+ * Copyright (C) 2008,2009 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -83,32 +84,34 @@ const char *  g_irepository_load_typelib  (GIRepository *repository,
 					   GIRepositoryLoadFlags flags,
 					   GError      **error);
 gboolean      g_irepository_is_registered (GIRepository *repository,
-					   const gchar  *namespace,
+					   const gchar  *namespace_,
 					   const gchar  *version);
 GIBaseInfo *  g_irepository_find_by_name  (GIRepository *repository,
-					   const gchar  *namespace,
+					   const gchar  *namespace_,
 					   const gchar  *name);
 GTypelib *    g_irepository_require       (GIRepository *repository,
-					   const gchar  *namespace,
+					   const gchar  *namespace_,
 					   const gchar  *version,
 					   GIRepositoryLoadFlags flags,
 					   GError      **error);
 gchar      ** g_irepository_get_dependencies (GIRepository *repository,
-					      const gchar  *namespace);
+					      const gchar  *namespace_);
 gchar      ** g_irepository_get_loaded_namespaces (GIRepository *repository);
 GIBaseInfo *  g_irepository_find_by_gtype (GIRepository *repository,
 					   GType         gtype);
 gint          g_irepository_get_n_infos   (GIRepository *repository,
-					   const gchar  *namespace);
+					   const gchar  *namespace_);
 GIBaseInfo *  g_irepository_get_info      (GIRepository *repository,
-					   const gchar  *namespace,
+					   const gchar  *namespace_,
 					   gint          index);
 const gchar * g_irepository_get_typelib_path   (GIRepository *repository,
-						const gchar  *namespace);
+						const gchar  *namespace_);
 const gchar * g_irepository_get_shared_library (GIRepository *repository,
-						const gchar  *namespace);
+						const gchar  *namespace_);
+const gchar * g_irepository_get_c_prefix (GIRepository *repository,
+                                          const gchar  *namespace_);
 const gchar * g_irepository_get_version (GIRepository *repository,
-					 const gchar  *namespace);
+					 const gchar  *namespace_);
 
 GOptionGroup * g_irepository_get_option_group (void);
 
@@ -141,6 +144,15 @@ typedef enum
 GQuark g_irepository_error_quark (void);
 
 
+/* Global utility functions */
+
+void gi_cclosure_marshal_generic (GClosure       *closure,
+                                  GValue         *return_gvalue,
+                                  guint           n_param_values,
+                                  const GValue   *param_values,
+                                  gpointer        invocation_hint,
+                                  gpointer        marshal_data);
+
 /* Types of objects registered in the repository */
 
 typedef enum 
@@ -170,14 +182,25 @@ typedef enum
 
 /* GIBaseInfo */
 
+typedef struct {
+  gpointer data;
+  gpointer data2;
+  gpointer data3;
+  gpointer data4;
+} GIAttributeIter;
+
 GIBaseInfo *           g_base_info_ref              (GIBaseInfo   *info);
 void                   g_base_info_unref            (GIBaseInfo   *info);
 GIInfoType             g_base_info_get_type         (GIBaseInfo   *info);
 const gchar *          g_base_info_get_name         (GIBaseInfo   *info);
 const gchar *          g_base_info_get_namespace    (GIBaseInfo   *info);
 gboolean               g_base_info_is_deprecated    (GIBaseInfo   *info);
-const gchar *          g_base_info_get_annotation   (GIBaseInfo   *info,
+const gchar *          g_base_info_get_attribute    (GIBaseInfo   *info,
                                                      const gchar  *name);
+gboolean               g_base_info_iterate_attributes (GIBaseInfo      *info,
+                                                       GIAttributeIter *iterator,
+                                                       char           **name,
+                                                       char          **value);
 GIBaseInfo *           g_base_info_get_container    (GIBaseInfo   *info);
 GTypelib *             g_base_info_get_typelib      (GIBaseInfo   *info);
 
@@ -217,6 +240,8 @@ typedef union
   guint64  v_uint64;
   gfloat   v_float;
   gdouble  v_double;
+  gshort   v_short;
+  gushort  v_ushort;
   gint     v_int;
   guint    v_uint;
   glong    v_long;
@@ -269,12 +294,26 @@ typedef enum  {
   GI_DIRECTION_INOUT
 } GIDirection;
 
+typedef enum {
+  GI_SCOPE_TYPE_INVALID, /* The argument is not of callback type */
+  GI_SCOPE_TYPE_CALL, /* The callback and associated user_data is only used during the
+                         call to this function */
+  GI_SCOPE_TYPE_ASYNC, /* The callback and associated user_data is 
+                          only used until the callback is invoked, and the callback 
+                          is invoked always exactly once. */
+  GI_SCOPE_TYPE_NOTIFIED /* The callback and and associated user_data is
+                            used until the caller is notfied via the destroy_notify */
+} GIScopeType;
+
 GIDirection            g_arg_info_get_direction          (GIArgInfo *info);
 gboolean               g_arg_info_is_dipper              (GIArgInfo *info);
 gboolean               g_arg_info_is_return_value        (GIArgInfo *info);
 gboolean               g_arg_info_is_optional            (GIArgInfo *info);
 gboolean               g_arg_info_may_be_null            (GIArgInfo *info);
 GITransfer             g_arg_info_get_ownership_transfer (GIArgInfo *info);
+GIScopeType            g_arg_info_get_scope              (GIArgInfo *info);
+gint                   g_arg_info_get_closure            (GIArgInfo *info);
+gint                   g_arg_info_get_destroy            (GIArgInfo *info);
 GITypeInfo *           g_arg_info_get_type               (GIArgInfo *info);
 
 
@@ -292,25 +331,27 @@ typedef enum {
   GI_TYPE_TAG_UINT32    =  7,
   GI_TYPE_TAG_INT64     =  8,
   GI_TYPE_TAG_UINT64    =  9,
-  GI_TYPE_TAG_INT       = 10,
-  GI_TYPE_TAG_UINT      = 11,
-  GI_TYPE_TAG_LONG      = 12,
-  GI_TYPE_TAG_ULONG     = 13,
-  GI_TYPE_TAG_SSIZE     = 14,
-  GI_TYPE_TAG_SIZE      = 15,
-  GI_TYPE_TAG_FLOAT     = 16,
-  GI_TYPE_TAG_DOUBLE    = 17,
-  GI_TYPE_TAG_TIME_T    = 18,
-  GI_TYPE_TAG_GTYPE     = 19,
-  GI_TYPE_TAG_UTF8      = 20,
-  GI_TYPE_TAG_FILENAME  = 21,
+  GI_TYPE_TAG_SHORT     = 10,
+  GI_TYPE_TAG_USHORT    = 11,
+  GI_TYPE_TAG_INT       = 12,
+  GI_TYPE_TAG_UINT      = 13,
+  GI_TYPE_TAG_LONG      = 14,
+  GI_TYPE_TAG_ULONG     = 15,
+  GI_TYPE_TAG_SSIZE     = 16,
+  GI_TYPE_TAG_SIZE      = 17,
+  GI_TYPE_TAG_FLOAT     = 18,
+  GI_TYPE_TAG_DOUBLE    = 19,
+  GI_TYPE_TAG_TIME_T    = 20,
+  GI_TYPE_TAG_GTYPE     = 21,
+  GI_TYPE_TAG_UTF8      = 22,
+  GI_TYPE_TAG_FILENAME  = 23,
   /* Non-basic types */
-  GI_TYPE_TAG_ARRAY     = 22,
-  GI_TYPE_TAG_INTERFACE = 23,
-  GI_TYPE_TAG_GLIST     = 24,
-  GI_TYPE_TAG_GSLIST    = 25,
-  GI_TYPE_TAG_GHASH     = 26,
-  GI_TYPE_TAG_ERROR     = 27
+  GI_TYPE_TAG_ARRAY     = 24,
+  GI_TYPE_TAG_INTERFACE = 25,
+  GI_TYPE_TAG_GLIST     = 26,
+  GI_TYPE_TAG_GSLIST    = 27,
+  GI_TYPE_TAG_GHASH     = 28,
+  GI_TYPE_TAG_ERROR     = 29
   /* Note - there is only room currently for 32 tags.
    * See docs/typelib-format.txt SimpleTypeBlob definition */
 } GITypeTag;
@@ -392,6 +433,7 @@ GIFunctionInfo *       g_struct_info_find_method   (GIStructInfo *info,
 						    const gchar *name);
 gsize                  g_struct_info_get_size      (GIStructInfo *info);
 gsize                  g_struct_info_get_alignment (GIStructInfo *info);
+gboolean               g_struct_info_is_gtype_struct (GIStructInfo *info);
 
 /* GIRegisteredTypeInfo */
 
@@ -432,9 +474,12 @@ GISignalInfo *         g_object_info_get_signal             (GIObjectInfo    *in
 gint                   g_object_info_get_n_vfuncs           (GIObjectInfo    *info);
 GIVFuncInfo *          g_object_info_get_vfunc              (GIObjectInfo    *info,
 							     gint            n);
+GIVFuncInfo *          g_object_info_find_vfunc             (GIObjectInfo *info,
+                                                             const gchar *name);
 gint                   g_object_info_get_n_constants        (GIObjectInfo    *info);
 GIConstantInfo *       g_object_info_get_constant           (GIObjectInfo    *info,
 							     gint            n);
+GIStructInfo *         g_object_info_get_class_struct       (GIObjectInfo    *info);
 
 							     
 /* GIInterfaceInfo */
@@ -456,10 +501,14 @@ GISignalInfo *         g_interface_info_get_signal          (GIInterfaceInfo *in
 gint                   g_interface_info_get_n_vfuncs        (GIInterfaceInfo *info);
 GIVFuncInfo *          g_interface_info_get_vfunc           (GIInterfaceInfo *info,
 							     gint        n);
+GIVFuncInfo *          g_interface_info_find_vfunc          (GIInterfaceInfo *info,
+                                                             const gchar *name);
 gint                   g_interface_info_get_n_constants     (GIInterfaceInfo *info);
 GIConstantInfo *       g_interface_info_get_constant        (GIInterfaceInfo *info,
 							     gint        n);
 
+GIStructInfo *         g_interface_info_get_iface_struct    (GIInterfaceInfo *info);
+
 
 /* GIPropertyInfo  */
 
@@ -486,6 +535,7 @@ typedef enum
 GIVFuncInfoFlags        g_vfunc_info_get_flags                   (GIVFuncInfo            *info);
 gint                    g_vfunc_info_get_offset                  (GIVFuncInfo            *info);
 GISignalInfo *          g_vfunc_info_get_signal                  (GIVFuncInfo            *info);
+GIFunctionInfo *        g_vfunc_info_get_invoker                 (GIVFuncInfo            *info);
 
 
 /* GIConstantInfo */
