@@ -161,31 +161,32 @@ g_typelib_check_sanity (void)
    * Everything else in the code however should be using sizeof().
    */
   
-  CHECK_SIZE (Header, 108);
+  CHECK_SIZE (Header, 112);
   CHECK_SIZE (DirEntry, 12);
   CHECK_SIZE (SimpleTypeBlob, 4);
   CHECK_SIZE (ArgBlob, 16);
   CHECK_SIZE (SignatureBlob, 8);
   CHECK_SIZE (CommonBlob, 8);
   CHECK_SIZE (FunctionBlob, 20);
+  CHECK_SIZE (CallbackBlob, 12);
   CHECK_SIZE (InterfaceTypeBlob, 4);
   CHECK_SIZE (ArrayTypeBlob, 8);
   CHECK_SIZE (ParamTypeBlob, 4);
   CHECK_SIZE (ErrorTypeBlob, 4);
   CHECK_SIZE (ErrorDomainBlob, 16);
   CHECK_SIZE (ValueBlob, 12);
-  CHECK_SIZE (FieldBlob, 12);
+  CHECK_SIZE (FieldBlob, 16);
   CHECK_SIZE (RegisteredTypeBlob, 16);
-  CHECK_SIZE (StructBlob, 24);
-  CHECK_SIZE (EnumBlob, 20);
-  CHECK_SIZE (PropertyBlob, 12);
-  CHECK_SIZE (SignalBlob, 12);
-  CHECK_SIZE (VFuncBlob, 16);
-  CHECK_SIZE (ObjectBlob, 32);
-  CHECK_SIZE (InterfaceBlob, 28);
-  CHECK_SIZE (ConstantBlob, 20);
-  CHECK_SIZE (AnnotationBlob, 12);
-  CHECK_SIZE (UnionBlob, 32);
+  CHECK_SIZE (StructBlob, 32);
+  CHECK_SIZE (EnumBlob, 24);
+  CHECK_SIZE (PropertyBlob, 16);
+  CHECK_SIZE (SignalBlob, 16);
+  CHECK_SIZE (VFuncBlob, 20);
+  CHECK_SIZE (ObjectBlob, 44);
+  CHECK_SIZE (InterfaceBlob, 40);
+  CHECK_SIZE (ConstantBlob, 24);
+  CHECK_SIZE (AttributeBlob, 12);
+  CHECK_SIZE (UnionBlob, 40);
 #undef CHECK_SIZE
 
   g_assert (size_check_ok);
@@ -313,25 +314,33 @@ validate_header (ValidateContext  *ctx,
 		   "Typelib size mismatch");
       return FALSE; 
     }
+  
+  /* This is a sanity check for a specific typelib; it
+   * prevents us from loading an incompatible typelib.
+   * 
+   * The hardcoded checks in g_typelib_check_sanity to
+   * protect against inadvertent or buggy changes to the typelib format
+   * itself.
+   */
 
-  if (header->entry_blob_size != 12 ||
-      header->function_blob_size != 20 ||
-      header->callback_blob_size != 12 ||
-      header->signal_blob_size != 12 ||
-      header->vfunc_blob_size != 16 ||
-      header->arg_blob_size != 16 ||
-      header->property_blob_size != 12 ||
-      header->field_blob_size != 12 ||
-      header->value_blob_size != 12 ||
-      header->constant_blob_size != 20 ||
-      header->error_domain_blob_size != 16 ||
-      header->annotation_blob_size != 12 ||
-      header->signature_blob_size != 8 ||
-      header->enum_blob_size != 20 ||
-      header->struct_blob_size != 24 ||
-      header->object_blob_size != 32 ||
-      header->interface_blob_size != 28 ||
-      header->union_blob_size != 32)
+  if (header->entry_blob_size != sizeof (DirEntry) ||
+      header->function_blob_size != sizeof (FunctionBlob) ||
+      header->callback_blob_size != sizeof (CallbackBlob) ||
+      header->signal_blob_size != sizeof (SignalBlob) ||
+      header->vfunc_blob_size != sizeof (VFuncBlob) ||
+      header->arg_blob_size != sizeof (ArgBlob) ||
+      header->property_blob_size != sizeof (PropertyBlob) ||
+      header->field_blob_size != sizeof (FieldBlob) ||
+      header->value_blob_size != sizeof (ValueBlob) ||
+      header->constant_blob_size != sizeof (ConstantBlob) ||
+      header->error_domain_blob_size != sizeof (ErrorDomainBlob) ||
+      header->attribute_blob_size != sizeof (AttributeBlob) ||
+      header->signature_blob_size != sizeof (SignatureBlob) ||
+      header->enum_blob_size != sizeof (EnumBlob) ||
+      header->struct_blob_size != sizeof (StructBlob) ||
+      header->object_blob_size != sizeof(ObjectBlob) ||
+      header->interface_blob_size != sizeof (InterfaceBlob) ||
+      header->union_blob_size != sizeof (UnionBlob))
     {
       g_set_error (error,
 		   G_TYPELIB_ERROR,
@@ -349,21 +358,21 @@ validate_header (ValidateContext  *ctx,
       return FALSE; 
     }
 
-  if (!is_aligned (header->annotations))
+  if (!is_aligned (header->attributes))
     {
       g_set_error (error,
 		   G_TYPELIB_ERROR,
 		   G_TYPELIB_ERROR_INVALID_HEADER,
-		   "Misaligned annotations");
+		   "Misaligned attributes");
       return FALSE; 
     }
 
-  if (header->annotations == 0 && header->n_annotations > 0)
+  if (header->attributes == 0 && header->n_attributes > 0)
     {
       g_set_error (error,
 		   G_TYPELIB_ERROR,
 		   G_TYPELIB_ERROR_INVALID_HEADER,
-		   "Wrong number of annotations");
+		   "Wrong number of attributes");
       return FALSE; 
     }
 
@@ -895,6 +904,8 @@ validate_constant_blob (GTypelib     *typelib,
     4, /* UINT32 */
     8, /* INT64 */
     8, /* UINT64 */
+    sizeof (gshort),
+    sizeof (gushort),
     sizeof (gint),
     sizeof (guint),
     sizeof (glong),
@@ -1435,7 +1446,9 @@ validate_object_blob (ValidateContext *ctx,
     {
       DirEntry *entry;
 
-      entry = g_typelib_get_dir_entry (typelib, blob->parent);
+      entry = get_dir_entry_checked (typelib, blob->parent, error);
+      if (!entry)
+        return FALSE;
       if (entry->blob_type != BLOB_TYPE_OBJECT &&
 	  (entry->local || entry->blob_type != 0))
 	{
@@ -1447,6 +1460,23 @@ validate_object_blob (ValidateContext *ctx,
 	}
     }
   
+  if (blob->gtype_struct != 0)
+    {
+      DirEntry *entry;
+
+      entry = get_dir_entry_checked (typelib, blob->gtype_struct, error);
+      if (!entry)
+        return FALSE;
+      if (entry->blob_type != BLOB_TYPE_STRUCT && entry->local)
+        {
+          g_set_error (error,
+                       G_TYPELIB_ERROR,
+                       G_TYPELIB_ERROR_INVALID_BLOB,
+                       "Class struct invalid type or not local");
+          return FALSE; 
+        }
+    }  
+  
   if (typelib->len < offset + sizeof (ObjectBlob) + 
             (blob->n_interfaces + blob->n_interfaces % 2) * 2 +
             blob->n_fields * sizeof (FieldBlob) +
@@ -1481,7 +1511,9 @@ validate_object_blob (ValidateContext *ctx,
 	  return FALSE; 
 	}
       
-      entry = g_typelib_get_dir_entry (typelib, iface);
+      entry = get_dir_entry_checked (typelib, iface, error);
+      if (!entry)
+        return FALSE;
 
       if (entry->blob_type != BLOB_TYPE_INTERFACE &&
 	  (entry->local || entry->blob_type != 0))
@@ -1830,13 +1862,13 @@ validate_directory (ValidateContext   *ctx,
 }
 
 static gboolean
-validate_annotations (ValidateContext *ctx, 
-		      GError       **error)
+validate_attributes (ValidateContext *ctx,
+		     GError       **error)
 {
   GTypelib *typelib = ctx->typelib;
   Header *header = (Header *)typelib->data;
 
-  if (header->size < header->annotations + header->n_annotations * sizeof (AnnotationBlob))
+  if (header->size < header->attributes + header->n_attributes * sizeof (AttributeBlob))
     {
       g_set_error (error,
 		   G_TYPELIB_ERROR,
@@ -1896,9 +1928,9 @@ g_typelib_validate (GTypelib     *typelib,
       return FALSE;
     }
 
-  if (!validate_annotations (&ctx, error))
+  if (!validate_attributes (&ctx, error))
     {
-      prefix_with_context (error, "annotations", &ctx);
+      prefix_with_context (error, "attributes", &ctx);
       return FALSE;
     }
 
@@ -2094,6 +2126,7 @@ g_typelib_get_namespace (GTypelib *typelib)
 
 /**
  * g_typelib_symbol:
+ * @typelib: the typelib
  * @symbol_name: name of symbol to be loaded
  * @symbol: returns a pointer to the symbol value
  *
