@@ -1,6 +1,7 @@
 # -*- Mode: Python -*-
 # GObject-Introspection - a framework for introspecting GObject libraries
 # Copyright (C) 2008  Johan Dahlin
+# Copyright (C) 2008, 2009 Red Hat, Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -35,6 +36,8 @@ TYPE_ANY = 'any'
 TYPE_BOOLEAN = 'boolean'
 TYPE_INT8 = 'int8'
 TYPE_UINT8 = 'uint8'
+TYPE_SHORT = 'short'
+TYPE_USHORT = 'ushort'
 TYPE_INT16 = 'int16'
 TYPE_UINT16 = 'uint16'
 TYPE_INT = 'int'
@@ -56,9 +59,10 @@ TYPE_FILENAME = 'filename'
 
 BASIC_GIR_TYPES = [TYPE_BOOLEAN, TYPE_INT8, TYPE_UINT8, TYPE_INT16,
                    TYPE_UINT16, TYPE_INT32, TYPE_UINT32, TYPE_INT64,
-                   TYPE_UINT64, TYPE_INT, TYPE_UINT, TYPE_LONG,
-                   TYPE_ULONG, TYPE_SSIZET, TYPE_SIZET, TYPE_FLOAT,
-                   TYPE_DOUBLE, TYPE_TIMET, TYPE_GTYPE]
+                   TYPE_UINT64, TYPE_SHORT, TYPE_USHORT, TYPE_INT,
+                   TYPE_UINT, TYPE_LONG, TYPE_ULONG, TYPE_SSIZET,
+                   TYPE_SIZET, TYPE_FLOAT, TYPE_DOUBLE, TYPE_TIMET,
+                   TYPE_GTYPE]
 GIR_TYPES = [TYPE_NONE, TYPE_ANY]
 GIR_TYPES.extend(BASIC_GIR_TYPES)
 GIR_TYPES.extend([TYPE_STRING, TYPE_FILENAME])
@@ -90,9 +94,9 @@ for name in GIR_TYPES:
 type_names['char'] = TYPE_INT8
 type_names['signed char'] = TYPE_INT8
 type_names['unsigned char'] = TYPE_UINT8
-type_names['short'] = TYPE_INT16
-type_names['signed short'] = TYPE_INT16
-type_names['unsigned short'] = TYPE_UINT16
+type_names['short'] = TYPE_SHORT
+type_names['signed short'] = TYPE_SHORT
+type_names['unsigned short'] = TYPE_USHORT
 type_names['int'] = TYPE_INT
 type_names['signed int'] = TYPE_INT
 type_names['signed'] = TYPE_INT
@@ -119,7 +123,7 @@ type_names['id'] = TYPE_ANY
 
 # Suppress some GLib names
 type_names['uchar'] = TYPE_UINT8
-type_names['ushort'] = TYPE_UINT16
+type_names['ushort'] = TYPE_USHORT
 type_names['size'] = TYPE_SIZET
 type_names['ssize'] = TYPE_SSIZET
 type_names['pointer'] = TYPE_ANY
@@ -134,8 +138,9 @@ default_array_types['uint8*'] = TYPE_UINT8
 default_array_types['utf8*'] = TYPE_STRING
 
 # These types, when seen by reference, are interpreted as out parameters
-default_out_types = (TYPE_INT, TYPE_UINT, TYPE_LONG, TYPE_ULONG,
-                     TYPE_FLOAT, TYPE_DOUBLE, TYPE_SIZET, TYPE_SSIZET)
+default_out_types = (TYPE_SHORT, TYPE_USHORT, TYPE_INT, TYPE_UINT,
+                     TYPE_LONG, TYPE_ULONG, TYPE_FLOAT, TYPE_DOUBLE,
+                     TYPE_SIZET, TYPE_SSIZET)
 
 
 def type_name_from_ctype(ctype):
@@ -146,13 +151,19 @@ class Node(object):
 
     def __init__(self, name=None):
         self.name = name
+        self.attributes = [] # (key, value)*
         self.deprecated = None
         self.deprecated_version = None
         self.version = None
 
+    def __cmp__(self, other):
+        return cmp(self.name, other.name)
+
     def __repr__(self):
         return '%s(%r)' % (self.__class__.__name__, self.name)
 
+    def remove_matching_children(self, pred):
+        pass
 
 class Namespace(Node):
 
@@ -165,6 +176,13 @@ class Namespace(Node):
         return '%s(%r, %r, %r)' % (self.__class__.__name__, self.name,
                                    self.version, self.nodes)
 
+    def remove_matching(self, pred):
+
+        def recursive_pred(node):
+            node.remove_matching_children(pred)
+            return pred(node)
+
+        self.nodes = filter(recursive_pred, self.nodes)
 
 class Include(Node):
 
@@ -191,15 +209,25 @@ class Include(Node):
     def __str__(self):
         return '%s-%s' % (self.name, self.version)
 
+class Callable(Node):
 
-class Function(Node):
-
-    def __init__(self, name, retval, parameters, symbol, throws=None):
+    def __init__(self, name, retval, parameters, throws):
         Node.__init__(self, name)
         self.retval = retval
         self.parameters = parameters
-        self.symbol = symbol
         self.throws = not not throws
+        self.doc = None
+
+    def __repr__(self):
+        return '%s(%r, %r, %r)' % (self.__class__.__name__,
+                                   self.name, self.retval,
+                                   self.parameters)
+
+class Function(Callable):
+
+    def __init__(self, name, retval, parameters, symbol, throws=None):
+        Callable.__init__(self, name, retval, parameters, throws)
+        self.symbol = symbol
         self.is_method = False
         self.doc = None
 
@@ -213,14 +241,18 @@ class Function(Node):
             if parameter.name == name:
                 return parameter
 
-    def __repr__(self):
-        return '%s(%r, %r, %r)' % (self.__class__.__name__,
-                                   self.name, self.retval,
-                                   self.parameters)
 
+class VFunction(Callable):
 
-class VFunction(Function):
-    pass
+    def __init__(self, name, retval, parameters, throws):
+        Callable.__init__(self, name, retval, parameters, throws)
+        self.invoker = None
+
+    @classmethod
+    def from_callback(cls, cb):
+        obj = cls(cb.name, cb.retval, cb.parameters[1:],
+                  cb.throws)
+        return obj
 
 
 class Type(Node):
@@ -272,7 +304,7 @@ class Map(Type):
         self.value_type = value_type
 
     def __repr__(self):
-        return 'Map(%r <%r,%r.)' % (self.name, self.key_type, self.value_type)
+        return 'Map(%r <%r,%r>)' % (self.name, self.key_type, self.value_type)
 
 
 class Alias(Node):
@@ -363,9 +395,13 @@ class Record(Node):
         self.symbol = symbol
         self.disguised = disguised
         self.doc = None
-        self.constructors = []
         self.methods = []
 
+    def remove_matching_children(self, pred):
+        self.fields = filter(pred, self.fields)
+        self.constructors = filter(pred, self.constructors)
+        self.methods = filter(pred, self.methods)
+
 # BW compat, remove
 Struct = Record
 
@@ -404,8 +440,10 @@ class Class(Node):
         Node.__init__(self, name)
         self.ctype = name
         self.parent = parent
+        self.glib_type_struct = None
         self.is_abstract = is_abstract
         self.methods = []
+        self.virtual_methods = []
         self.static_methods = []
         self.interfaces = []
         self.constructors = []
@@ -413,6 +451,12 @@ class Class(Node):
         self.fields = []
         self.doc = None
 
+    def remove_matching_children(self, pred):
+        self.methods = filter(pred, self.methods)
+        self.constructors = filter(pred, self.constructors)
+        self.properties = filter(pred, self.properties)
+        self.fields = filter(pred, self.fields)
+
     def __repr__(self):
         return '%s(%r, %r, %r)' % (
             self.__class__.__name__,
@@ -425,6 +469,8 @@ class Interface(Node):
         Node.__init__(self, name)
         self.parent = parent
         self.methods = []
+        self.virtual_methods = []
+        self.glib_type_struct = None
         self.properties = []
         self.fields = []
         self.prerequisites = []
@@ -479,6 +525,16 @@ class Callback(Node):
         self.throws = False
         self.doc = None
 
+    def get_parameter_index(self, name):
+        for i, parameter in enumerate(self.parameters):
+            if parameter.name == name:
+                return i
+
+    def get_parameter(self, name):
+        for parameter in self.parameters:
+            if parameter.name == name:
+                return parameter
+
     def __repr__(self):
         return 'Callback(%r, %r, %r)' % (
             self.name, self.retval, self.parameters)
