@@ -146,7 +146,8 @@ class SourceType(object):
 
 
 class SourceSymbol(object):
-    __members__ = ['const_int', 'const_string', 'ident', 'type', 'base_type']
+    __members__ = ['const_int', 'const_double', 'const_string', 'ident',
+                   'type', 'base_type']
 
     def __init__(self, scanner, symbol):
         self._scanner = scanner
@@ -162,6 +163,10 @@ class SourceSymbol(object):
     def const_int(self):
         return self._symbol.const_int
 
+    @property
+    def const_double(self):
+        return self._symbol.const_double
+
     @property
     def const_string(self):
         return self._symbol.const_string
@@ -179,6 +184,10 @@ class SourceSymbol(object):
         if self._symbol.base_type is not None:
             return SourceType(self._scanner, self._symbol.base_type)
 
+    @property
+    def source_filename(self):
+        return self._symbol.source_filename
+
 
 class SourceScanner(object):
 
