@@ -21,10 +21,21 @@ typedef const gint* (*AnnotationCallback) (const gint *in);
  */
 typedef GList* (*AnnotationListCallback) (GList *in);
 
+/**
+ * AnnotationNotifyFunc:
+ * @data: (closure): The user data
+ *
+ * This is a callback with a 'closure' argument that is not named
+ * 'user_data' and hence has to be annotated.
+ */
+typedef void (*AnnotationNotifyFunc) (gpointer data);
+
 /**
  * AnnotationObject:
  *
  * This is an object used to test annotations.
+ *
+ * Attributes: (org.example.Test cows)
  */
 typedef struct _AnnotationObject          AnnotationObject;
 typedef struct _AnnotationObjectClass     AnnotationObjectClass;
@@ -67,6 +78,8 @@ gint     annotation_object_calleesowns  (AnnotationObject *object,
 					 GObject          **toown2);
 GList*   annotation_object_get_strings  (AnnotationObject *object);
 GHashTable*annotation_object_get_hash   (AnnotationObject *object);
+void     annotation_object_with_voidp   (AnnotationObject *object,
+					 void             *data);
 GSList*  annotation_object_get_objects  (AnnotationObject *object);
 
 void     annotation_object_use_buffer   (AnnotationObject *object,
@@ -101,6 +114,13 @@ void     annotation_object_set_data3    (AnnotationObject *object,
                                          gsize             length);
 
 GObject* annotation_object_do_not_use   (AnnotationObject *object);
+void     annotation_object_watch        (AnnotationObject *object,
+                                         AnnotationForeachFunc func,
+                                         gpointer user_data);
+void     annotation_object_watch_full   (AnnotationObject *object,
+                                         AnnotationForeachFunc func,
+                                         gpointer user_data,
+                                         GDestroyNotify destroy);
 
 void     annotation_init                (int              *argc, 
 					 char           ***argv);
@@ -109,6 +129,14 @@ void     annotation_versioned           (void);
 char **  annotation_string_zero_terminated (void);
 void     annotation_string_zero_terminated_out (char ***out);
 
+void     annotation_object_extra_annos (AnnotationObject *object);
+
+void     annotation_custom_destroy (AnnotationCallback callback,
+                                    AnnotationNotifyFunc destroy,
+                                    gpointer data);
+char *   annotation_get_source_file (void);
+void     annotation_set_source_file (const char *fname);
+
 /**
  * AnnotationStruct:
  *
