@@ -1,6 +1,5 @@
-#define FOO_SUCCESS_INT 0x1138
-
 #include "foo.h"
+#include "girepository.h"
 
 /* A hidden type not exposed publicly, similar to GUPNP's XML wrapper
    object */
@@ -10,8 +9,8 @@ int foo_init_argv (int argc, char **argv);
 int foo_init_argv_address (int *argc, char ***argv);
 void foo_private_function (FooObject *foo);
 void foo_test_unsigned (unsigned int uint);
-void foo_interface_do_foo (FooInterface *self);
-void foo_do_foo (FooInterface *self);
+void foo_interface_do_foo (FooInterface *self, int x);
+void foo_do_foo (FooInterface *self, int x);
 int foo_enum_method (FooEnumType foo_enum);
 FooHidden * foo_hidden_copy (const FooHidden *boxed);
 void foo_hidden_free (FooHidden *boxed);
@@ -49,9 +48,9 @@ foo_interface_get_type (void)
   return object_type;
 }
 
-void foo_interface_do_foo (FooInterface *self)
+void foo_interface_do_foo (FooInterface *self, int x)
 {
-  FOO_INTERFACE_GET_INTERFACE(self)->do_foo (self);
+  FOO_INTERFACE_GET_INTERFACE(self)->do_foo (self, x);
 }
 
 enum {
@@ -75,6 +74,16 @@ foo_foo_interface_init (gpointer         g_iface,
   iface->do_foo = foo_do_foo;
 }
 
+enum {
+  SUBIFACE_DESTROY_EVENT,
+  SUBIFACE_LAST_SIGNAL
+};
+
+static void
+foo_sub_interface_class_init (gpointer g_class, gpointer class_data);
+
+static guint foo_subiface_signals[SUBIFACE_LAST_SIGNAL] = { 0 };
+
 GType
 foo_sub_interface_get_type (void)
 {
@@ -85,7 +94,7 @@ foo_sub_interface_get_type (void)
       object_type = g_type_register_static_simple (G_TYPE_INTERFACE,
                                                    "FooSubInterface",
                                                    sizeof (FooSubInterfaceIface),
-                                                   NULL, 0, NULL, 0);
+                                                   foo_sub_interface_class_init, 0, NULL, 0);
 
       g_type_interface_add_prerequisite (object_type, FOO_TYPE_INTERFACE);
     }
@@ -93,6 +102,20 @@ foo_sub_interface_get_type (void)
   return object_type;
 }
 
+static void
+foo_sub_interface_class_init (gpointer g_class, gpointer class_data)
+{
+  foo_subiface_signals[SUBIFACE_DESTROY_EVENT] =
+    g_signal_new ("destroy-event", FOO_TYPE_SUBINTERFACE,
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (FooSubInterfaceIface, destroy_event),
+                  NULL, NULL,
+                  gi_cclosure_marshal_generic,
+                  G_TYPE_NONE,
+                  0,
+                  G_TYPE_NONE);
+}
+
 void foo_sub_interface_do_bar (FooSubInterface *self)
 {
   FOO_SUBINTERFACE_GET_INTERFACE(self)->do_bar (self);
@@ -193,7 +216,7 @@ foo_object_take_all (FooObject *object, int x, ...)
 }
 
 void
-foo_do_foo (FooInterface *self)
+foo_do_foo (FooInterface *self, int x)
 {
 
 
@@ -216,6 +239,22 @@ foo_object_dup_name (FooObject *object)
   return g_strdup ("foo");
 }
 
+/**
+ * foo_object_read:
+ * @object: obj
+ * @offset: offset
+ * @length: length
+ *
+ * Read some stuff.
+ *
+ * Virtual: read_fn
+ */
+void
+foo_object_read (FooObject *object, int offset, int length)
+{
+
+}
+
 G_DEFINE_ABSTRACT_TYPE (FooSubobject, foo_subobject, FOO_TYPE_OBJECT);
 
 static void
@@ -230,6 +269,20 @@ foo_subobject_init (FooSubobject *object)
 
 }
 
+/**
+ * foo_object_get_default:
+ *
+ * This function is intended to match clutter_stage_get_default which
+ * uses a C sugar return type.
+ *
+ * Return value: (type FooSubobject): The global #FooSubobject
+ */
+FooObject *
+foo_object_get_default ()
+{
+  return NULL;
+}
+
 int foo_init (void)
 {
   return FOO_SUCCESS_INT;
@@ -420,7 +473,6 @@ foo_hidden_free (FooHidden *boxed)
   g_slice_free (FooHidden, boxed);
 }
 
-
 GType
 foo_hidden_get_type (void)
 {
@@ -455,3 +507,75 @@ foo_error_quark (void)
   return g_quark_from_static_string ("foo-error-quark");
 }
 
+void
+foo_some_variant (guint x, va_list args)
+{
+}
+
+GType foo_tile_handler_get_type(void);
+
+/* This setup of a totally hidden parent class matches
+ * http://bugzilla.gnome.org/show_bug.cgi?id=561360
+ */
+#define FOO_TYPE_TILE_HANDLER        (foo_tile_handler_get_type ())
+#define FOO_TILE_HANDLER(object)     (G_TYPE_CHECK_INSTANCE_CAST ((object), FOO_TYPE_TILE_HANDLER, FooTileHandler))
+#define FOO_IS_TILE_HANDLER(object)  (G_TYPE_CHECK_INSTANCE_TYPE ((object), FOO_TYPE_TILE_HANDLER))
+
+struct _FooTileHandler
+{
+  FooObject parent_instance;
+};
+
+
+struct _FooTileHandlerClass
+{
+  FooObjectClass parent_class;
+};
+
+typedef struct _FooTileHandler          FooTileHandler;
+typedef struct _FooTileHandlerClass     FooTileHandlerClass;
+
+G_DEFINE_TYPE(FooTileHandler, foo_tile_handler, FOO_TYPE_OBJECT);
+
+static void
+foo_tile_handler_class_init (FooTileHandlerClass *klass)
+{
+
+}
+
+static void
+foo_tile_handler_init (FooTileHandler *object)
+{
+
+}
+
+struct _FooBuffer
+{
+  FooTileHandler parent_instance;
+};
+
+struct _FooBufferClass
+{
+  FooTileHandlerClass parent_class;
+};
+
+G_DEFINE_TYPE(FooBuffer, foo_buffer, FOO_TYPE_TILE_HANDLER);
+
+static void
+foo_buffer_class_init (FooBufferClass *klass)
+{
+
+}
+
+static void
+foo_buffer_init (FooBuffer *object)
+{
+
+}
+
+void
+foo_buffer_some_method (FooBuffer *buffer)
+{
+}
+
+#define FOO_DEFINE_SHOULD_NOT_BE_EXPOSED "should not be exposed"
